package com.hz.pm.api.projectdeclared.manage;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.toolkit.Assert;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.hz.pm.api.common.model.constant.BizConst;
import com.hz.pm.api.common.statemachine.util.StateMachineUtil;
import com.hz.pm.api.projectdeclared.model.dto.OperationReq;
import com.hz.pm.api.projectdeclared.model.entity.Operation;
import com.hz.pm.api.projectdeclared.model.vo.OperationVO;
import com.hz.pm.api.projectdeclared.service.IOperationService;
import com.hz.pm.api.projectlib.model.entity.Project;
import com.hz.pm.api.projectlib.model.enumeration.ProjectStatusEnum;
import com.hz.pm.api.projectlib.service.IProjectService;
import com.hz.pm.api.user.security.model.UserInfoDetails;
import com.hz.pm.api.user.util.LoginUserUtil;
import com.ningdatech.basic.function.VUtils;
import com.ningdatech.basic.util.CollUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;

/**
 * @Classname OperationManage
 * @Description
 * @Date 2023/7/31 14:48
 * @Author PoffyZhang
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class OperationManage {

    private final IOperationService operationService;

    private final IProjectService projectService;

    private final StateMachineUtil stateMachineUtil;

    /**
     * 获取实施详情
     *
     * @param projectId \
     * @return \
     */
    public List<OperationVO> detail(Long projectId) {
        Project project = projectService.getNewProject(projectId);
        Assert.notNull(project, "项目不存在!");

        List<Operation> operations = operationService.list(Wrappers.lambdaQuery(Operation.class)
                .eq(Operation::getProjectCode, project.getProjectCode()));

        return CollUtils.convert(operations, w -> BeanUtil.copyProperties(w, OperationVO.class));
    }

    @Transactional(rollbackFor = Exception.class)
    public String pushOperation(OperationReq req) {
        UserInfoDetails user = LoginUserUtil.loginUserDetail();

        Project project = projectService.getNewProject(req.getProjectId());
        Assert.notNull(project, "项目不存在!");

        //首先要判断 项目当前状态 是不是 待开工状态
        VUtils.isTrue(!ProjectStatusEnum.OPERATION.eq(project.getStatus()) ||
                        !ProjectStatusEnum.PROJECT_APPROVED.eq(project.getStage()))
                .throwMessage("提交失败 该项目不是 待开工或者已立项阶段");

        Operation pre = operationService.getOne(Wrappers.lambdaQuery(Operation.class)
                .eq(Operation::getProjectCode, project.getProjectCode())
                .eq(Operation::getBidId, req.getBidId())
                .last(BizConst.LIMIT_1));

        LocalDateTime now = LocalDateTime.now();
        Operation curr = BeanUtil.copyProperties(req, Operation.class);
        if (Objects.nonNull(pre)) {
            curr.setId(pre.getId());
        } else {
            curr.setCreateOn(now);
            curr.setCreateBy(user.getUsername());
        }
        curr.setProjectCode(project.getProjectCode());
        curr.setUpdateOn(now);
        curr.setUpdateBy(user.getUsername());
        operationService.saveOrUpdate(curr);
        return curr.getProjectCode();
    }
}
